@@ -41,14 +41,11 @@
 #define WARN_DEPRECATED_NEW           117
 #define WARN_DEPRECATED_EXCEPT_TM     118
 #define WARN_DEPRECATED_IGNORE_TM     119
-#define WARN_DEPRECATED_OPTC          120
-#define WARN_DEPRECATED_NAME          121
 
 /* -- Preprocessor -- */
 
 #define WARN_PP_MISSING_FILE          201
 #define WARN_PP_EVALUATION            202
-#define WARN_PP_INCLUDEALL_IMPORTALL  203
 
 /* -- C/C++ Parser -- */
 
@@ -71,9 +68,6 @@
 #define WARN_PARSE_TEMPLATE_SP_UNDEF  317
 #define WARN_PARSE_TEMPLATE_AMBIG     318
 #define WARN_PARSE_NO_ACCESS          319
-#define WARN_PARSE_EXPLICIT_TEMPLATE  320
-#define WARN_PARSE_BUILTIN_NAME       321
-#define WARN_PARSE_REDUNDANT          322
 
 #define WARN_IGNORE_OPERATOR_NEW        350   /* new */
 #define WARN_IGNORE_OPERATOR_DELETE     351   /* delete */
@@ -121,7 +115,6 @@
 #define WARN_IGNORE_OPERATOR_UAND       393   /* & */
 #define WARN_IGNORE_OPERATOR_NEWARR     394   /* new [] */
 #define WARN_IGNORE_OEPRATOR_DELARR     395   /* delete [] */
-#define WARN_IGNORE_OEPRATOR_REF        396   /* operator *() */
 
 /* 394-399 are reserved */
 
@@ -146,8 +139,6 @@
 #define WARN_TYPEMAP_VAR_UNDEF        466
 #define WARN_TYPEMAP_TYPECHECK        467
 #define WARN_TYPEMAP_THROW            468
-#define WARN_TYPEMAP_DIRECTORIN_UNDEF  469
-#define WARN_TYPEMAP_THREAD_UNSAFE    470
 
 /* -- General code generation -- */
 
@@ -164,61 +155,23 @@
 #define WARN_LANG_OVERLOAD_KEYWORD    511
 #define WARN_LANG_OVERLOAD_CONST      512
 #define WARN_LANG_CLASS_UNNAMED       513
-#define WARN_LANG_DIRECTOR_VDESTRUCT  514
-#define WARN_LANG_DISCARD_CONST       515
-#define WARN_LANG_OVERLOAD_IGNORED    516
-#define WARN_LANG_DIRECTOR_ABSTRACT   517
 
 /* -- Reserved (600-799) -- */
 
 /* -- Language module specific warnings (800 - 999) -- */
 
-#define WARN_RUBY_WRONG_NAME                  801
-#define WARN_RUBY_MULTIPLE_INHERITANCE        802
+#define WARN_RUBY_WRONG_NAME            801
+#define WARN_RUBY_MULTIPLE_INHERITANCE  802
 
-#define WARN_JAVA_TYPEMAP_JNI_UNDEF           810
-#define WARN_JAVA_TYPEMAP_JTYPE_UNDEF         811
-#define WARN_JAVA_TYPEMAP_JSTYPE_UNDEF        812
-#define WARN_JAVA_MULTIPLE_INHERITANCE        813
-#define WARN_JAVA_TYPEMAP_GETCPTR_UNDEF       814
-#define WARN_JAVA_TYPEMAP_CLASSMOD_UNDEF      815
-#define WARN_JAVA_TYPEMAP_JAVABODY_UNDEF      816
-#define WARN_JAVA_TYPEMAP_JAVAOUT_UNDEF       817
-#define WARN_JAVA_TYPEMAP_JAVAIN_UNDEF        818
-#define WARN_JAVA_TYPEMAP_JAVADIRECTORIN_UNDEF    819
-#define WARN_JAVA_TYPEMAP_JAVADIRECTOROUT_UNDEF   820
-#define WARN_JAVA_COVARIANT_RET               822
+#define WARN_JAVA_TYPEMAP_JNI_UNDEF     810
+#define WARN_JAVA_TYPEMAP_JTYPE_UNDEF   811
+#define WARN_JAVA_TYPEMAP_JSTYPE_UNDEF  812
+#define WARN_JAVA_MULTIPLE_INHERITANCE  813
+#define WARN_JAVA_TYPEMAP_GETCPTR_UNDEF 814
+#define WARN_JAVA_TYPEMAP_CLASSMOD_UNDEF    815
+#define WARN_JAVA_TYPEMAP_PTRCONSTMOD_UNDEF 816
 
-/* please leave 810-829 free for Java */
 
-#define WARN_CSHARP_TYPEMAP_CTYPE_UNDEF       830
-#define WARN_CSHARP_TYPEMAP_CSTYPE_UNDEF      831
-#define WARN_CSHARP_TYPEMAP_CSWTYPE_UNDEF     832
-#define WARN_CSHARP_MULTIPLE_INHERITANCE      833
-#define WARN_CSHARP_TYPEMAP_GETCPTR_UNDEF     834
-#define WARN_CSHARP_TYPEMAP_CLASSMOD_UNDEF    835
-#define WARN_CSHARP_TYPEMAP_CSBODY_UNDEF      836
-#define WARN_CSHARP_TYPEMAP_CSOUT_UNDEF       837
-#define WARN_CSHARP_TYPEMAP_CSIN_UNDEF        838
-#define WARN_CSHARP_TYPEMAP_CSDIRECTORIN_UNDEF    839
-#define WARN_CSHARP_TYPEMAP_CSDIRECTOROUT_UNDEF   840
-#define WARN_CSHARP_COVARIANT_RET             842
-
-/* please leave 830-849 free for C# */
-
-#define WARN_MODULA3_TYPEMAP_TYPE_UNDEF        850
-#define WARN_MODULA3_TYPEMAP_GETCPTR_UNDEF     851
-#define WARN_MODULA3_TYPEMAP_CLASSMOD_UNDEF    852
-#define WARN_MODULA3_TYPEMAP_PTRCONSTMOD_UNDEF 853
-#define WARN_MODULA3_TYPEMAP_MULTIPLE_RETURN   854
-#define WARN_MODULA3_MULTIPLE_INHERITANCE      855
-#define WARN_MODULA3_TYPECONSTRUCTOR_UNKNOWN   856
-#define WARN_MODULA3_UNKNOWN_PRAGMA            857
-#define WARN_MODULA3_BAD_ENUMERATION           858
-#define WARN_MODULA3_DOUBLE_ID                 859
-#define WARN_MODULA3_BAD_IMPORT                860
-
-/* please leave 850-869 free for Modula 3 */
 
 
 
